#include "ads1110.h"
#include "esphome/core/log.h"
#include "esphome/core/hal.h"

namespace esphome {
namespace ads1110 {
namespace {

const char *const TAG = "ads1110";

constexpr uint8_t ST = 0x80;
constexpr uint8_t SC = 0x10;
constexpr uint8_t DR_240SPS = 0 << 2;
constexpr uint8_t DR_60SPS = 1 << 2;
constexpr uint8_t DR_30SPS = 2 << 2;
constexpr uint8_t DR_15SPS = 3 << 2;
constexpr uint8_t PGA_1 = 0 << 0;
constexpr uint8_t PGA_2 = 1 << 0;
constexpr uint8_t PGA_4 = 2 << 0;
constexpr uint8_t PGA_8 = 3 << 0;
constexpr uint8_t PGA_MASK = 3 << 0;

constexpr double kVref = 2.048;

}  // namespace

void ADS1110Sensor::setup() {
  uint8_t config = DR_15SPS | PGA_2;
  write(&config, 1);
}

void ADS1110Sensor::dump_config() {
  LOG_SENSOR("", "ADS1110", this);
  LOG_I2C_DEVICE(this);
  LOG_UPDATE_INTERVAL(this);
}

void ADS1110Sensor::update() {
  uint8_t data[3];
  if (read(data, sizeof(data)) == i2c::NO_ERROR && (data[2] & ST) == 0) {
    int16_t code = static_cast<int16_t>((data[0] << 8) | data[1]);
    double scale = kVref / 32768;
    scale /= 1 << (data[2] & PGA_MASK);
    this->publish_state(code * scale);
  }
}

}  // namespace ads1110
}  // namespace esphome
