import json
import sys
import glob
import os
import re
import hashlib
import shutil

VARIANTS = {
    "ESP32S3": "ESP32-S3",
    "modwifi": "ESP8266",
}


def main():
    os.makedirs("dist", exist_ok=True)

    for path in glob.glob(".esphome/build/*/src/main.cpp"):
        fields = {}
        with open(path) as f:
            for line in f:
                line = line.strip()
                match = re.match(r"//\s+(\w+):\s+'?([^']+)", line)
                if not match:
                    continue
                fields[match.group(1)] = match.group(2)

        name = fields.get("project_name", None)
        version = fields.get("project_version", None)
        variant = fields.get("variant", None)
        if not variant:
            variant = VARIANTS.get(fields.get("board", None), None)

        if None in (name, version, variant):
            continue

        configuration = path.split("/")[2]
        ota = (
            f".esphome/build/{configuration}/.pioenvs/{configuration}/firmware.ota.bin"
        )
        if not os.path.exists(ota):
            continue

        with open(ota, "r+b") as f:
            md5 = hashlib.md5()
            md5.update(f.read())

        target = f"{name}-{version}.bin"
        shutil.copy(ota, f"dist/{target}")
        manifest = {
            "name": name,
            "version": version,
            "builds": [
                {
                    "chipFamily": VARIANTS.get(variant, variant),
                    "ota": {
                        "md5": md5.hexdigest(),
                        "path": target,
                    },
                }
            ],
        }
        with open(f"dist/{name}.json", "w") as f:
            json.dump(manifest, f, indent=4)


if __name__ == "__main__":
    main()
